
unit GameStatePlay;

interface

uses Classes,
  CastleUIState, CastleComponentSerialize, CastleUIControls, CastleControls,
  CastleKeysMouse, CastleViewport, CastleScene, CastleVectors, StrUtils, Dialogs,
  CastleWindow,CastleUtils, CastleGLImages, CastleFilesUtils,
  CastleSoundEngine, CastleTimeUtils, CastleColors,
  CastleRectangles, CastleFonts, CastleGLUtils, Generics.Defaults,
  Generics.Collections, CastleOnScreenMenu, CastleDownload, CastleSceneCore,
  CastleCameras, CastleTransform,
  CastleImages, X3DNodes;


  type
    { Main "playing game" state, where most of the game logic takes place. }
    TStatePlay = class(TUIState)
    private
      Mouse: TCastleScene;
      PlayerScene: TCastleTransform;
      X, Y, S: single;
      StandIdleAnim: TCastleScene;
      WalkLeftAnim, WalkRightAnim, WalkFrontAnim, WalkBackAnim: TCastleScene;
      Label1: TCastleLabel;
      procedure MakeMousePhysics;
      procedure MouseCollisionEnter(const CollisionDetails: TPhysicsCollisionDetails);
      procedure PlayerScenePhysics;

      public
      constructor Create(AOwner: TComponent); override;
      procedure Start; override;
      procedure Update(const SecondsPassed: Single; var HandleInput: Boolean); override;
      function Press(const Event: TInputPressRelease): Boolean; override;
    end;


var
  StatePlay: TStatePlay;
  Stand, WalkLeft, WalkRight, WalkFront, WalkBack: boolean;

implementation

uses SysUtils, Math,
  GameStateMenu;

{ TStatePlay ----------------------------------------------------------------- }

var
  MainViewport: TCastleViewport;

Procedure TStatePlay.MouseCollisionEnter(const CollisionDetails: TPhysicsCollisionDetails);
begin
  if CollisionDetails.OtherTransform <> nil then
   begin
     if CollisionDetails.OtherTransform = PlayerScene then Label1.Caption := 'YESSS!'; // .Terminate;
   end;
end;

Procedure TStatePlay.MakeMousePhysics;
 var
  RBody: TRigidBody;
  Size: TVector3;
  Collider: TBoxCollider;
  begin
    RBody := TRigidBody.Create(Mouse);
    RBody.Dynamic:= true;
    RBody.Setup2D;
    RBody.OnCollisionEnter:= @MouseCollisionEnter;

    Collider := TBoxCollider.Create(RBody);

    Size := Mouse.LocalBoundingBox.Size;
    Size.Z := Max(0.1, Size.Z);
    Collider.Size := Size;
    Mouse.RigidBody := RBody;
end;

Procedure TStatePlay.PlayerScenePhysics;
 var
  RBody: TRigidBody;
  Size: TVector3;
  Collider: TBoxCollider;
begin
  RBody := TRigidBody.Create(PlayerScene);
  RBody.Dynamic:= true;
  RBody.Setup2D;

  Collider := TBoxCollider.Create(RBody);

  Size := PlayerScene.LocalBoundingBox.Size;
  Size.Z := Max(0.1, Size.Z);
  Collider.Size := Size;
  PlayerScene.RigidBody := RBody;
end;

procedure StopWalk;
begin
  Stand := false;
  WalkLeft := false;
  WalkRight := false;
  WalkFront := false;
  WalkBack := false;
  StatePlay.StandIdleAnim.Exists := false;
  StatePlay.WalkFrontAnim.Exists := false;
  StatePlay.WalkBackAnim.Exists := false;
  StatePlay.WalkLeftAnim.Exists := false;
  StatePlay.WalkRightAnim.Exists := false;
end;

Procedure GoLeft;
begin
  StopWalk;
  with StatePlay.WalkLeftAnim do
  begin
    Exists := true;
    WalkLeft := true;
    PlayAnimation('johnwalkleft', true);
  end;
end;

Procedure StandLeft;
begin
  StopWalk;
  with StatePlay.StandIdleAnim do
  begin
    Exists := true;
    WalkLeft := false;
    Stand := true;
    PlayAnimation('johnstandleft', true);
  end;
end;

Procedure GoRight;
begin
  StopWalk;
   with StatePlay.WalkRightAnim do
  begin
    Exists := true;
    WalkRight := true;
    Stand := false;
    PlayAnimation('johnwalkright', true);
  end;
end;

Procedure StandRight;
begin
  StopWalk;
  with StatePlay.StandIdleAnim do
  begin
    Exists := true;
    WalkRight := false;
    Stand := true;
    PlayAnimation('johnstandright', true);
  end;
end;

Procedure GoBack;
begin
  StopWalk;
  with StatePlay.WalkBackAnim do
  begin
    Exists := true;
    WalkBack := true;
    Stand := false;
    PlayAnimation('johnwalkback', true);
  end;
end;

Procedure StandBack;
begin
  StopWalk;
  with StatePlay.StandIdleAnim do
  begin
    Exists := true;
    WalkBack := false;
    Stand := true;
    PlayAnimation('johnstandback', true);
  end;
end;

Procedure GoFront;
begin
  StopWalk;
  with StatePlay.WalkFrontAnim do
  begin
  Exists := true;
    WalkFront := true;
    Stand := false;
    PlayAnimation('johnwalkfront', true);
  end;
end;

Procedure StandFront;
begin
 StopWalk;
 with StatePlay.StandIdleAnim do
 begin
 Exists := true;
  WalkFront := false;
  Stand := true;
 PlayAnimation('johnstandfront', true);
  end;
 end;

constructor TStatePlay.Create(AOwner: TComponent);
begin
  inherited;
end;

procedure TStatePlay.Start;
begin
  inherited;
   DesignUrl := 'castle-data:/westbeach-gamestateplay.castle-user-interface';
   MainViewport := DesignedComponent ('Viewport') as TCastleViewport;
   Mouse := DesignedComponent ('Mouse') as TCastleScene;
   PlayerScene := DesignedComponent ('PlayerScene') as TCastleTransform;
   StandIdleAnim := DesignedComponent ('StandIdleAnim') as TCastleScene;
   WalkLeftAnim := DesignedComponent ('WalkLeftAnim') as TCastleScene;
   WalkRightAnim := DesignedComponent ('WalkRightAnim') as TCastleScene;
   WalkFrontAnim := DesignedComponent ('WalkFrontAnim') as TCastleScene;
   WalkBackAnim := DesignedComponent ('WalkBackAnim') as TCastleScene;
   Label1 := DesignedComponent ('Label1') as TCastleLabel;

 StandFront; // initial animation of playerscene
 S := 1;

 MakeMousePhysics;
// PlayerScenePhysics;

end;

procedure TStatePlay.Update(const SecondsPassed: Single; var HandleInput: Boolean);
begin
  MainViewport.Cursor := mcNone;
  Mouse.Translation := Vector3(Mainviewport.PositionTo2DWorld(Container.MousePosition, true), 9);

 //if Mouse.BoundingBox.RectangleXY.Collides(StatePlay.PlayerScene.BoundingBox.RectangleXY.Grow(-20, -20)) then Label1.Caption := 'collision!' else Label1.Caption := ' ';

 if WalkLeft then
  begin
    X := X - 8;
    PlayerScene.Translation := Vector3(X, Y, 1);
  end;

 if WalkRight then
 begin
   X := X + 8;
   PlayerScene.Translation := Vector3(X, Y, 1);
 end;

 if WalkFront then
 begin
   Y := Y - 3;
   S := S + 0.005;
   PlayerScene.Translation := Vector3(X, Y, 1);
   PlayerScene.Scale := Vector3(S, S, S);
 end;

 if WalkBack then
 begin
   Y := Y + 3;
   S := S - 0.005;
  PlayerScene.Translation := Vector3(X, Y, 1);
  PlayerScene.Scale := Vector3(S, S, S);
 end;
end;


function TStatePlay.Press(const Event: TInputPressRelease): Boolean;
begin
  Result := inherited;
  if Result then Exit; // allow the ancestor to handle keys

  if Event.IsKey(keyArrowLeft) then
  begin
    if Stand then GoLeft else StandLeft;
  end;

  if Event.IsKey(keyArrowRight) then
  begin
    if Stand then GoRight else StandRight;
   end;

  if Event.IsKey(keyArrowUp) then
  begin
    if Stand then GoBack else StandBack;
  end;

   if Event.IsKey(keyArrowDown) then
  begin
    if Stand then GoFront else StandFront;
  end;

  if Event.IsKey(keyEscape) then
  begin
    TUIState.Current := StateMenu;
    Exit(true);
  end;
end;

end.
